import { getImmutableRecordKey } from "./utils.js";
export class MutableModelAPI {
    static async get(key, dbContext) {
        const record = await dbContext.records.select({ _key: key });
        if (record === null) {
            return null;
        }
        for (const [propertyName, relation] of Object.entries(dbContext.relations)) {
            record[propertyName] = (await relation.selectAll({ _source: key })).map(({ _target }) => _target);
        }
        return record;
    }
    static async set(key, value, options = {}, dbContext) {
        let version = null;
        const existingVersion = await dbContext.records.selectVersion({ _key: key });
        const existingTombstone = await dbContext.tombstones.select({ _key: key });
        if (dbContext.resolve !== undefined) {
            version = options.version ?? null;
            if (existingVersion !== null && existingVersion._version !== null) {
                if (version === null) {
                    return;
                }
                else if (dbContext.resolve.lessThan({ version }, { version: existingVersion._version })) {
                    return;
                }
            }
            if (existingTombstone !== null && existingTombstone._version !== null) {
                if (version === null) {
                    return;
                }
                else if (dbContext.resolve.lessThan({ version }, { version: existingTombstone._version })) {
                    return;
                }
            }
        }
        if (existingTombstone !== null) {
            await dbContext.tombstones.delete({ _key: key });
        }
        if (existingVersion === null) {
            await dbContext.records.insert({ _key: key, _version: version, value });
        }
        else {
            await dbContext.records.update({ _key: key, _version: version, value });
            for (const relation of Object.values(dbContext.relations)) {
                await relation.deleteAll({ _source: key });
            }
        }
        for (const [propertyName, relation] of Object.entries(dbContext.relations)) {
            const targets = value[propertyName];
            if (!Array.isArray(targets)) {
                throw new TypeError(`${dbContext.model.name}/${propertyName} must be string[]`);
            }
            for (const target of targets) {
                if (typeof target !== "string") {
                    throw new TypeError(`${dbContext.model.name}/${propertyName} must be string[]`);
                }
                await relation.create({ _source: key, _target: target });
            }
        }
    }
    static async delete(key, options = {}, dbContext) {
        let version = null;
        const previous = await dbContext.records.selectVersion({ _key: key });
        const tombstone = await dbContext.tombstones.select({ _key: key });
        if (dbContext.resolve !== undefined) {
            version = options.version ?? null;
            if (previous !== null && previous._version !== null) {
                if (version === null || dbContext.resolve.lessThan({ version }, { version: previous._version })) {
                    return;
                }
            }
            if (tombstone !== null && tombstone._version !== null) {
                if (version === null || dbContext.resolve.lessThan({ version }, { version: tombstone._version })) {
                    return;
                }
            }
        }
        await dbContext.records.delete({ _key: key });
        for (const relation of Object.values(dbContext.relations)) {
            await relation.deleteAll({ _source: key });
        }
        if (dbContext.resolve !== undefined && version !== null) {
            if (tombstone === null) {
                await dbContext.tombstones.insert({ _key: key, _version: version });
            }
            else {
                await dbContext.tombstones.update({ _key: key, _version: version });
            }
        }
    }
    static iterate(dbContext) {
        return dbContext.records.iterate({});
    }
    static async selectAll(dbContext) {
        return dbContext.records.selectAll({});
    }
    static async query(query, dbContext) {
        return dbContext.records.query(query);
    }
    static async count(dbContext) {
        return dbContext.records.count();
    }
}
export class ImmutableModelAPI {
    static async add(value, { namespace } = {}, dbContext) {
        const key = getImmutableRecordKey(value, { namespace });
        const existingRecord = await dbContext.records.select({ _key: key });
        if (!existingRecord) {
            await dbContext.records.insert({ _key: key, value });
            for (const [propertyName, relation] of Object.entries(dbContext.relations)) {
                const targets = value[propertyName];
                if (!Array.isArray(targets)) {
                    throw new TypeError(`${dbContext.model.name}/${propertyName} must be string[]`);
                }
                for (const target of targets) {
                    if (typeof target !== "string") {
                        throw new TypeError(`${dbContext.model.name}/${propertyName} must be string[]`);
                    }
                    await relation.create({ _source: key, _target: target });
                }
            }
        }
        return key;
    }
    static async remove(key, dbContext) {
        const existingRecord = await dbContext.records.select({ _key: key });
        if (existingRecord !== null) {
            await dbContext.records.delete({ _key: key });
            for (const relation of Object.values(dbContext.relations)) {
                await relation.deleteAll({ _source: key });
            }
        }
    }
    static async get(key, dbContext) {
        const record = await dbContext.records.select({ _key: key });
        if (record === null) {
            return null;
        }
        for (const [propertyName, relation] of Object.entries(dbContext.relations)) {
            record[propertyName] = (await relation.selectAll({ _source: key })).map(({ _target }) => _target);
        }
        return record;
    }
    static iterate(dbContext) {
        return dbContext.records.iterate({});
    }
    static async selectAll(dbContext) {
        return dbContext.records.selectAll({});
    }
    static async query(query, dbContext) {
        return dbContext.records.query(query);
    }
    static async count(dbContext) {
        return dbContext.records.count();
    }
}
